<?php

/**
 * The service type base class.
 */
class Provision_Service_Certificate extends Provision_Service {
  public $service = 'Certificate';

  // Whether the certificate can generate a default server certificate.
  public $can_generate_default = FALSE;

  /**
   * Return service-specific configuration options for help.
   *
   * @return
   *   array('option' => 'description')
   */
  static function option_documentation() {
    return array(
      'https_cert_source' => 'site: The path to the certificate file to use for this site.',
      'https_cert_key_source' => 'site: The path to the key file to use for this site.',
      'https_cert_chain_source' => 'site: The path to the key file to use for this site.',
    );
  }

  /**
   * Called on provision-verify.
   */
  function verify() {
    #$this->create_config(d()->type);
    #$this->parse_configs();
  }

  /**
   * Initialize this class, including option handling.
   */
  function init_site() {

    /**
     * Configurable values.
     */
    $source_path = $this->get_source_path($this->context->uri);
    $this->context->setProperty('https_cert_source', "{$source_path}/openssl.crt");
    $this->context->setProperty('https_cert_key_source', "{$source_path}/openssl.key");
    $this->context->setProperty('https_cert_chain_source', "{$source_path}/openssl_chain.crt");

    /**
     * Non configurable values.
     */
    $target_path = "{$this->server->http_ssld_path}/{$this->context->uri}";
    $this->context->https_cert = "{$target_path}/openssl.crt";
    $this->context->https_cert_key = "{$target_path}/openssl.key";
    $this->context->https_cert_chain = "{$target_path}/openssl_chain.crt";

  }
//  /**
//   * Add the needed properties to the site context.
//   */
//  static function subscribe_site($context) {
//    $context->setProperty('https_cert_source');
//    $context->setProperty('https_cert_key_source');
//    $context->setProperty('https_cert_chain_source');
//  }

  /**
   * PUBLIC API
   */

  /**
   * Return the path where we'll generate our certificates.
   */
  function get_source_path($https_key) {
    // Default to the ~/config/ssl.d directory.
    return "{$this->server->ssld_path}/{$https_key}";
  }

  /**
   * Retrieve an array containing the actual files for this https_key.
   *
   * Always attempt to generate new certificates.  The upstream script should
   * recognize non-expired ones, and leave them in place.  So it checks for
   * us.  We don't need to check which ones are still valid.
   */
  function get_certificates($https_key) {
    $this->generate_certificates($https_key);
    $certs = $this->get_certificate_paths($https_key);

    // If a certificate chain file exists, add it.
    if (!empty(d()->https_cert_chain_source) && provision_file()->exists(d()->https_cert_chain_source)->status()) {
        $certs['https_cert_chain_source'] = d()->https_cert_chain_source;
        $certs['https_cert_chain'] = d()->https_cert_chain ?? '';
        drush_log('Chain file was found at ' . d()->https_cert_chain_source, 'ok');
    }
    else {
        drush_log('No chain file found at ' . d()->https_cert_chain_source, 'ok');
    }
    return $certs;
  }

  /**
   * Retrieve an array containing source and target paths for this https_key.
   */
  function get_certificate_paths($https_key) {

    $certs = array();
    $certs['https_cert_key_source'] = $this->context->https_cert_key_source;
    $certs['https_cert_key'] =  $this->context->https_cert_key;
    $certs['https_cert_source'] = $this->context->https_cert_source;
    $certs['https_cert'] = $this->context->https_cert;

    return $certs;
  }

  /**
   * Generate a self-signed certificate for the provided key.
   */
  function generate_certificates($https_key) {
    // This is a dummy implementation. We should probably move this into an
    // interface.
    return TRUE;
  }

  /**
   * Commonly something like running the restart_cmd or sending SIGHUP to a process.
   */
  function parse_configs() {
    return TRUE;
  }

  /**
   * Generate a site specific configuration file.
   */
  function create_site_config() {
    return TRUE;
  }

  /**
   * Remove an existing site configuration file.
   */
  function delete_site_config() {
    return TRUE;
  }

  /**
   * Add a new platform specific configuration file.
   */
  function create_platform_config() {
    return TRUE;
  }

  /**
   * Remove an existing platform configuration file.
   */
  function delete_platform_config() {
    return TRUE;
  }

  /**
   * Create a new server specific configuration file.
   */
  function create_server_config() {
    return TRUE;
  }

  /**
   * Remove an existing server specific configuration file.
   */
  function delete_server_config() {
    return TRUE;
  }
}
